#include <cv.h>
#include <highgui.h>
#include <stdio.h>

int background_subtraction_histogram(IplImage * input_image, int width, int height, int cam_num, 
                                     int buf_num, IplImage * gray_background)
{
        double sum_box[10][352];
        double sum_box_max[10][352];
        int max_id[352];
        double max_val[352];

        IplImage *dif_image;
        IplImage *gray;
        IplImage *red;
        IplImage *green;
        IplImage *blue;
        IplImage *mask;
        CvHistogram *rhist;
        CvHistogram *ghist;
        CvHistogram *bhist;
       
        /* --- Initialize buffers --- */ 
        dif_image = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
        gray = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
        red = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
        green = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
        blue = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
        mask = cvCreateImage( cvSize(width,height), IPL_DEPTH_8U, 1 );

        int hdims = 32;
        float hranges_arr[] = {0, 255};
        float* hranges = hranges_arr;

        rhist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );
        ghist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );
        hdims = 16;
        bhist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );

        int n, m;
        CvScalar s;

        for(n = 0; n < height; n++)
        for(m = 0; m < width; m++)
        {
          s.val[0]= 255;
          cvSet2D(mask,n,m,s);
        }
 
        /* --- background subtraction --- */
        char temp[100];
        cvCvtColor(input_image, gray, CV_BGR2GRAY);
        cvSplit( input_image, red, green, blue, 0 );
        double test_sum = 0;
        for(m = 0; m < 100; m ++)
        for(n = 0; n < 100; n ++)
        {
           CvScalar s = cvGet2D(gray, n, m);
           test_sum += s.val[0];
        }
        
        if (test_sum == 0)
        {

            char fname[200];
            sprintf(fname, "objects_info_%d.txt", buf_num + 2);
            
            FILE *fp = fopen(fname, "wt");   
            fprintf(fp, "%d\n", 1);
            for(int i = 0; i < 4; i++)
                   fprintf(fp, "%d %d %d %d", 50, 80, 60, 120);
            for(int i = 0; i < 64; i++)
                   fprintf(fp, "%f ", 10.0);
            fclose(fp);

            return 1;
        }
        cvAbsDiff(gray, gray_background, dif_image);
        cvThreshold(dif_image, dif_image, 25, 255, CV_THRESH_BINARY);
        cvSmooth( dif_image, dif_image, CV_MEDIAN);

        /*for(m = 0; m < width; m++)
        for(n = 0; n < height; n++)
        {
          CvScalar s = cvGet2D(gray_background, n, m);
          CvScalar g = cvGet2D(gray, n, m);
          CvScalar b;
          b.val[0] = s.val[0] * 0.9 + g.val[0] * 0.1;
          cvSet2D(gray_background, n, m, b);
        }*/

        /* --- locate human like objects --- */
        for(n = 0; n < 10; n++)
        {
           double scale = 1/10.0 * double(n) + 1.0;
           int w = floor(30 * scale/2);
           int h = floor(100 * scale/2);
           double sum_temp[320];
           for(m = 0; m < 320; m++)
           {  sum_temp[m] = 0;
              for(int i = 120 - h; i < 120 + h; i++)
              {
                 CvScalar s = cvGet2D(dif_image, i, m);
                 sum_temp[m] += s.val[0];
              }
              sum_box[n][m] = 0;
           }

           double xsum = 0;
           for(m = 0; m < 2*w+1; m++)
              xsum += sum_temp[m];
           sum_box[n][w] = xsum;

           for(m = w+1; m < 320 - w; m++)
           {
              xsum = xsum - sum_temp[m-w-1] + sum_temp[m+w];
              sum_box[n][m] = xsum * scale / (4*h*w);
           }
        }

        for(m = 0; m < 320; m++)
        {
           double max = -1e10;
           for(n = 0; n < 10; n ++)
           {
               if( sum_box[n][m] > max)
               {
                   max = sum_box[n][m];
                  max_id[m] = n;
               }
           }
           max_val[m] = max;
        }

        int num_of_boxes = 0;
        int cc[500];
        int rr[500];
        for( m = 30; m < 320-30; m++)
        {
            if(max_val[m] >= max_val[m-1] &&
               max_val[m] >= max_val[m+1] &&
               max_val[m] > 50 && num_of_boxes < 500)
             {
                cc[num_of_boxes] = m;
                rr[num_of_boxes] = max_id[m];
                num_of_boxes++;
             }
        }

        /* --- compute and save color histograms in bounding boxes --- */ 

        char fname[200];
        sprintf(fname, "objects_info_%d.txt", buf_num + 2);
        FILE *fp = fopen(fname, "wt");

        if(num_of_boxes == 0)
        {
            fprintf(fp, "1\n");
            fprintf(fp, "1 2 1 2 ");
            for(m = 0; m < 64; m ++ )
               fprintf(fp, "10.0 ");
            fprintf(fp, "\n");
        }
        else
        {
          fprintf(fp, "%d\n", num_of_boxes);
          for(n = 0; n < num_of_boxes; n++)
         {
           double scale = 1/10.0 * rr[n] + 1;
           int w = floor(30.0 * scale/2);
           int h = floor(100.0 * scale/2);
           CvPoint pt1, pt2;
           pt1.x = cc[n] - w;
           if (pt1.x < 0) pt1.x = 0;
           pt1.y = 120 - h;
           pt2.x = cc[n] + w;
           if (pt2.x > 320) pt2.x = 320;
           pt2.y = 120 + h;

           double sv = 0;
           int i, j;
           int top = pt1.y;
           int down = pt2.y;
           for(i = top; i <= down; i++)
           {
               sv = 0;
               for(j = pt1.x; j <= pt2.x; j++)
               {
                   CvScalar s = cvGet2D(dif_image, i, j);
                   sv += s.val[0];
               }
               if (sv > 0) break;
           }
 
           top = i;

           for(i = down; i >= top; i--)
           {
               sv = 0;
               for(j = pt1.x; j <= pt2.x; j++)
               {
                   CvScalar s = cvGet2D(dif_image, i, j);
                   sv += s.val[0];
               }
               if (sv > 0) break;
           }

           down = i;

           pt1.y = top;
           pt2.y = down;

           CvRect selection = cvRect(pt1.x, pt1.y, (pt2.x - pt1.x + 1), (pt2.y - pt1.y + 1));

           cvSetImageROI( red, selection );
           cvSetImageROI( mask, selection );
           cvCalcHist( &red, rhist, 0, mask );
           cvNormalizeHist( rhist, 0.333 );
           cvResetImageROI( red );
           cvResetImageROI( mask );

           cvSetImageROI( green, selection );
           cvSetImageROI( mask, selection );
           cvCalcHist( &green, ghist, 0, mask );
           cvNormalizeHist( ghist, 0.333 );
           cvResetImageROI( green );
           cvResetImageROI( mask );

           cvSetImageROI( blue, selection );
           cvSetImageROI( mask, selection );
           cvCalcHist( &blue, bhist, 0, mask );
           cvNormalizeHist( bhist, 0.333 );
           cvResetImageROI( blue );
           cvResetImageROI( mask );

           fprintf(fp, "%d %d %d %d ", pt1.x, pt2.x, pt1.y, pt2.y);

           for(m = 0; m < 24; m ++)
           {
               double hist_val = cvQueryHistValue_1D( rhist, m );
               fprintf(fp, "%f ", hist_val);
           }

           for(m = 0; m < 24; m ++)
           {
               double hist_val = cvQueryHistValue_1D( ghist, m );
               fprintf(fp, "%f ", hist_val);
           }

           for(m = 0; m < 16; m ++)
           {
               double hist_val = cvQueryHistValue_1D( bhist, m );
               fprintf(fp, "%f ", hist_val);
           }

           fprintf(fp, "\n");
          }
         }

         fclose(fp);

         /* --- clean the buffers --- */
         cvReleaseImage(&gray);
         cvReleaseImage(&red);
         cvReleaseImage(&green);
         cvReleaseImage(&blue);

         cvReleaseHist(&rhist);
         cvReleaseHist(&ghist);
         cvReleaseHist(&bhist);

         return 0;
}
   

