//////////////////////////////////////////////////////
// Camera Class 
//////////////////////////////////////////////////////
// Hao Jiang
// March 2007, UBC
/////////////////////////////////////////////////////
//
// Current version supports Axis 206 network camera and
// video4linux video capturing card        
// 
///////////////////////////////////////////////////////
#ifndef CAM_H
#define CAM_H    

#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <linux/videodev.h>
#include <stdlib.h>
#include <fcntl.h>

class Camera
{
    protected:  
     int width;
     int height;

    public:
      Camera(int w, int h) 
      { width = w; height = h;
      }
 
      virtual void capture_image(IplImage* ) {};
};

class NetworkCamera : public Camera
{
     int sockfd;
     public:
       NetworkCamera(int w, int h, char *addr) : Camera(w, h)
       {
         int ret = init_cam(sockfd, addr, w, h);
         if (ret) 
         {
            printf("camera\n");
            exit(ret);
         }
       }

       void capture_image(IplImage* image) 
       {     
            get_image(sockfd, image);
       }
      
     private:
       int readline(int fd, char* linebuf, int maxlength);
       int init_cam(int &sockfd, char *addr, int width, int height);
       void  get_image(int sockfd, IplImage * image);    
};
          

class ImageSequence : public Camera
{
     int start;
     int end;
     char prefix[100];
     int cur_num;

     public:
       ImageSequence(char *name, int w, int h, int s, int e) : Camera(w, h), start(s), end(e), cur_num(s)
       {
           strcpy(prefix, name);
       }

       void capture_image(IplImage* image)
       {
            char name[100];
            sprintf(name, "%s%d.jpg", prefix, cur_num);
            cur_num++;
            if(cur_num > end) cur_num = end;
            IplImage *img = cvLoadImage(name);
            cvCopy(img, image);
            cvReleaseImage(&img);
       }

};



#endif 


